package betterwithmods.module.hardcore.crafting;

import betterwithmods.common.BWMRecipes;
import betterwithmods.common.BWOreDictionary;
import betterwithmods.module.Feature;
import betterwithmods.module.gameplay.CrucibleRecipes;
import com.google.common.collect.Sets;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPostInitializationEvent;
import net.minecraftforge.oredict.OreDictionary;
import net.minecraftforge.oredict.ShapedOreRecipe;
import net.minecraftforge.oredict.ShapelessOreRecipe;

import java.util.Arrays;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Created by primetoxinz on 4/20/17.
 */
public class HCOres extends Feature {

	private static boolean oreNuggetSmelting, dustNuggetSmelting, fixVanillaRecipes;
	private static Set<String> oreExclude, dustExclude;
	private static int oreProductionCount, dustProductionCount;

	@Override
	public void setupConfig() {
		oreNuggetSmelting = loadPropBool("Ore to Nugget Smelting", "Make Ores (oredict ore.* )smelt into nuggets instead of ingots", true);

		oreExclude = Arrays.stream(loadPropStringList("Ore Exclude", "Oredictionary entries to exclude from ore to nugget smelting. Remove the prefix of the oredictionary. example 'oreIron' would be just 'iron' ", new String[0])).map(String::toLowerCase).collect(Collectors.toSet());
		dustExclude = Arrays.stream(loadPropStringList("Dust Exclude", "Oredictionary entries to exclude from dust to nugget smelting  Remove the prefix of the oredictionary. example 'dustIron' would be just 'iron'", new String[0])).map(String::toLowerCase).collect(Collectors.toSet());

		dustNuggetSmelting = loadPropBool("Dust to Nugget Smelting", "Make Dusts ( oredict dust.* ) smelt into nuggets instead of ingots", true);
		fixVanillaRecipes = loadPropBool("Fix Vanilla Recipes", "Make certain recipes cheaper to be more reasonable with nugget smelting, including Compass, Clock, and Bucket", true);

		oreProductionCount = loadPropInt("Ore Production Count", "Number of Materials returned from Smelting an Ore", 1);
		dustProductionCount = loadPropInt("Dust Production Count", "Number of Materials returned from Smelting a Dust", 1);
	}

	@Override
	public String getFeatureDescription() {
		return "Makes Ores only smelt into a single nugget, making it much harder to create large amounts of metal";
	}

	@Override
	public boolean requiresMinecraftRestartToEnable() {
		return true;
	}

	@Override
	public void init(FMLInitializationEvent event) {
		if (fixVanillaRecipes) {
			addHardcoreRecipe(new ShapedOreRecipe(null, Items.field_151111_aL, " N ", "NRN", " N ", 'N', "nuggetIron", 'R', "dustRedstone").setRegistryName(new ResourceLocation("minecraft", "compass")));
			addHardcoreRecipe(new ShapedOreRecipe(null, Items.field_151113_aN, " N ", "NQN", " N ", 'N', "nuggetGold", 'Q', "gemQuartz").setRegistryName(new ResourceLocation("minecraft", "clock")));
			addHardcoreRecipe(new ShapedOreRecipe(null, Items.field_151133_ar, "N N", "N N", "NNN", 'N', "nuggetIron").setRegistryName(new ResourceLocation("minecraft", "bucket")));
			addHardcoreRecipe(new ShapelessOreRecipe(null, Items.field_151033_d, Items.field_151145_ak, "nuggetIron").setRegistryName(new ResourceLocation("minecraft", "flint_and_steel")));
		}
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da, 7), new Object[] { new ItemStack(Items.field_151133_ar) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da, 7), new Object[] { new ItemStack(Items.field_151131_as) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da, 7), new Object[] { new ItemStack(Items.field_151117_aB) });

		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da, 4), new Object[] { new ItemStack(Items.field_151148_bJ) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da, 4), new Object[] { new ItemStack(Items.field_151111_aL) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151074_bl, 4), new Object[] { new ItemStack(Items.field_151113_aN) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da), new Object[] { new ItemStack(Blocks.field_150479_bC, 2, 0) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_191525_da), new Object[] { new ItemStack(Items.field_151033_d) });
	}

	@Override
	public void disabledInit(FMLInitializationEvent event) {
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151042_j, 3), new Object[] { new ItemStack(Items.field_151133_ar) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151042_j), new Object[] { new ItemStack(Blocks.field_150479_bC, 2, 0) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151042_j), new Object[] { new ItemStack(Items.field_151033_d, 1, OreDictionary.WILDCARD_VALUE) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151042_j, 4), new Object[] { new ItemStack(Items.field_151111_aL) });
		CrucibleRecipes.addStokedCrucibleRecipe(new ItemStack(Items.field_151043_k, 4), new Object[] { new ItemStack(Items.field_151113_aN) });
	}

	@Override
	public void postInit(FMLPostInitializationEvent event) {
		Set<String> oreExcludes = Sets.union(oreExclude, Sets.newHashSet("oreDiamond"));
		if (oreNuggetSmelting) {
			for (BWOreDictionary.Ore ore : BWOreDictionary.oreNames) {
				if (!oreExcludes.contains(ore.getOre())) {
					Optional<ItemStack> nugget = BWOreDictionary.nuggetNames.stream().filter(o -> o.getSuffix().equals(ore.getSuffix())).flatMap(o -> o.getOres().stream()).findFirst();
					if (nugget.isPresent()) {
						ItemStack n = nugget.get().func_77946_l();
						n.func_190920_e(oreProductionCount);
						//Remove all furnace recipes with dust
						ore.getOres().forEach(BWMRecipes::removeFurnaceRecipe);
						//Add dust -> nugget smelting recipe
						ore.getOres().forEach(s -> BWMRecipes.addFurnaceRecipe(s, n));
					}
				}
			}
		}
		Set<String> dustExcludes = Sets.union(dustExclude, Sets.newHashSet("dustDiamond"));
		if (dustNuggetSmelting) {
			for (BWOreDictionary.Ore dust : BWOreDictionary.dustNames) {
				if (!dustExcludes.contains(dust.getOre())) {
					Optional<ItemStack> nugget = BWOreDictionary.nuggetNames.stream().filter(o -> o.getSuffix().equals(dust.getSuffix())).flatMap(o -> o.getOres().stream()).findFirst();
					if (nugget.isPresent()) {
						ItemStack n = nugget.get().func_77946_l();
						n.func_190920_e(dustProductionCount);
						//Remove all furnace recipes with dust
						dust.getOres().forEach(BWMRecipes::removeFurnaceRecipe);
						//Add dust -> nugget smelting recipe
						dust.getOres().forEach(s -> BWMRecipes.addFurnaceRecipe(s, n));
					}
				}
			}
		}
	}

}
